<?php

class Cloudsme_Prototype_Admin
{
    private $plugin_name;

    private $version;

    public function __construct($plugin_name, $version)
    {

        $this->plugin_name = $plugin_name;
        $this->version = $version;

    }

    public function create_admin_menu_page()
    {

        add_submenu_page('options-general.php', 'Cloudsme prototype options', 'Cloudsme options', 'manage_options', 'cloudsme-prototype.php', array($this, 'show_proto_admin_settings'));

    }

    public function settings_selection_script() {
        wp_enqueue_script( 'settings_selection', plugin_dir_url( __FILE__ ) . 'js/cloudsme-prototype-admin.js', array( 'jquery' ) );
        wp_localize_script( 'settings_selection', 'PT_Ajax', array(
                'ajaxurl'   => admin_url( 'admin-ajax.php' ),
                'nextNonce' => wp_create_nonce( 'myajax-next-nonce' )
            )
        );
    }

    public function settings_selection_func() {
        $nonce = $_POST['nextNonce'];
        if ( ! wp_verify_nonce( $nonce, 'myajax-next-nonce' ) ) {
            die ( 'Nonce verification failed' );
        }

        $command = $_POST['command'];
        $response_data = array();

        if($command) {
            switch ($command) {

                case 'selected_soft':
                    $software_id = $_POST['software_id'];
                    $response_data['executables'] = $this -> getAvailableExecutables($software_id);
                    $response_data['resources'] = $this -> getAvailableResources($software_id);

                break;

                case 'selected_resource':
                    $response_data['regions'] = $this -> getAvailableRegions($_POST['software_id'],$_POST['resource_id']);

                    break;

                case 'selected_region':
                    $response_data['itypes'] = $this -> getAvailableITypes($_POST['software_id'], $_POST['region_id']);

                    break;

                case 'cred_test':
                    //before selecting options ensure that credentials are correct

                    $response_data['result'] =  $_POST;

                    break;
            }
        } else {
            $response_data = 'unknown command';
        }

        $response = json_encode($response_data );
        header( "Content-Type: application/json" );
        echo $response;
        exit;
    }

    private function authorize_admin() {
        //attempting to log in with current settings
        $url = get_option('o_platform_url');
        $username = get_option('o_email');
        $password = get_option('o_password');

        if(empty($url) || empty($username) || empty($password)) {
            update_option('o_admin_authorized', 'false');

        } else {
            $url .= "/users/current.xml";
            $process = curl_init($url);
            curl_setopt($process, CURLOPT_HEADER, 1);
            curl_setopt($process, CURLOPT_USERPWD, $username . ":" . $password);
            curl_setopt($process, CURLOPT_TIMEOUT, 30);
            curl_setopt($process, CURLOPT_RETURNTRANSFER, TRUE);
            curl_setopt($process, CURLOPT_VERBOSE, 1);

            $return = curl_exec($process);

            $header_size = curl_getinfo($process, CURLINFO_HEADER_SIZE);
            $header = substr($return, 0, $header_size);
            $body = substr($return, $header_size);

            curl_close($process);

            $xml = simplexml_load_string($body);

            if (!empty($xml) && !empty($xml->id)) {
                update_option('o_admin_authorized', 'true');
            } else {
                update_option('o_admin_authorized', 'false');
            }
        }
    }

    private function unset_tmp_ids() {
        if(isset($_SESSION['init_soft_id'])) unset($_SESSION['init_soft_id']);
        if(isset($_SESSION['init_exec_id'])) unset($_SESSION['init_exec_id']);
        if(isset($_SESSION['init_res_id'])) unset($_SESSION['init_res_id']);
        if(isset($_SESSION['init_reg_id'])) unset($_SESSION['init_reg_id']);
        if(isset($_SESSION['init_itype_id'])) unset($_SESSION['init_itype_id']);
        if(isset($_SESSION['init_input_type'])) unset($_SESSION['init_input_type']);
    }

    public function show_proto_admin_settings()
    {

        if (!current_user_can('manage_options')) {
            wp_die('<p>You do not have sufficient permissions to access this page.</p>');
        }

        //authorize admin to interact with platform

        $this -> authorize_admin();
        $this -> unset_tmp_ids();

        if (get_option('o_admin_authorized') == 'false') {
            //access to options before admin's authorization

            ?>
            <div class="tw-bs">
                <h1>CloudSME Platform integration settings</h1>
                <form action="options.php" method="post">
                    <?php wp_nonce_field('update-options'); ?>
                    <div id="csme_admin_msg">
                        <p>It is impossible to access platform with provided credentials</p>
                    </div>
                    <table class="form-table">
                        <tr valign="top">
                            <th scope="row">URL</th>
                            <td><input class="cred-selector" type="text" id="o_platform_url" name="o_platform_url"
                                       value="<?php echo get_option('o_platform_url'); ?>"/></td>
                        </tr>
                        <tr valign="top">
                            <th scope="row">Login</th>
                            <td><input class="cred-selector" type="text" id="o_email" name="o_email"
                                       value="<?php echo get_option('o_email'); ?>"/></td>
                        </tr>

                        <tr valign="top">
                            <th scope="row">Password</th>
                            <td><input class="cred-selector" type="password" id="o_password" name="o_password"
                                       value="<?php echo get_option('o_password'); ?>"/></td>
                        </tr>

                    </table>

                    <input type="hidden" name="action" value="update"/>
                    <input type="hidden" name="page_options"
                           value="o_platform_url,o_email,o_password"/>
                    <p class="submit">
                        <input type="submit" class="button-primary" value="<?php _e('Save Changes') ?>"/>
                    </p>
                </form>
            </div>
            <?php

        } else {

                ?>

                <div class="tw-bs">
                    <h1>CloudSME Platform integration settings</h1>
                    <form action="options.php" method="post">
                        <?php wp_nonce_field('update-options'); ?>

                        <table class="form-table">
                            <tr valign="top">
                                <th scope="row">URL</th>
                                <td><input class="cred-selector" type="text" id="o_platform_url" name="o_platform_url"
                                           value="<?php echo get_option('o_platform_url'); ?>"/></td>
                            </tr>

                            <tr valign="top">
                                <th scope="row">Login</th>
                                <td><input class="cred-selector" type="text" id="o_email" name="o_email"
                                           value="<?php echo get_option('o_email'); ?>"/></td>
                            </tr>

                            <tr valign="top">
                                <th scope="row">Password</th>
                                <td><input class="cred-selector" type="password" id="o_password" name="o_password"
                                           value="<?php echo get_option('o_password'); ?>"/></td>
                            </tr>

                            <tr valign="top">

                                <?php $this->constructSoftwareList(); ?>
                                <input type="hidden" id="o_software" name="o_software"
                                       value="<?php echo (isset($_SESSION['init_soft_id'])) ? $_SESSION['init_soft_id'] : get_option('o_software'); ?>"/>

                                <?php $this->constructExecutableList(); ?>
                                <input type="hidden" id="o_executable" name="o_executable"
                                       value="<?php echo (isset($_SESSION['init_exec_id'])) ? $_SESSION['init_exec_id'] : get_option('o_executable'); ?>"/>

                                <?php $this->constructResourseList(); ?>
                                <input type="hidden" id="o_resource" name="o_resource"
                                       value="<?php echo (isset($_SESSION['init_res_id'])) ? $_SESSION['init_res_id'] : get_option('o_resource'); ?>"/>

                                <?php $this->constructRegionList(); ?>
                                <input type="hidden" id="o_region" name="o_region"
                                       value="<?php echo (isset($_SESSION['init_reg_id'])) ? $_SESSION['init_reg_id'] : get_option('o_region'); ?>"/>

                                <?php $this->constructInstanceTypeList(); ?>
                                <input type="hidden" id="o_instance_type" name="o_instance_type"
                                       value="<?php echo (isset($_SESSION['init_itype_id'])) ? $_SESSION['init_itype_id'] : get_option('o_instance_type'); ?>"/>

                            <tr>
                                <th>Input type</th>
                                <td>
                                    <select class="id-selector" id="o_input_type" name="o_input_type"
                                            data-storedvalue="<?php echo (isset($_SESSION['init_input_type'])) ? $_SESSION['init_input_type'] : get_option('o_input_type'); ?>">
                                        <option value="stdin">stdin</option>
                                        <option value="input">input</option>
                                    </select>
                                </td>
                            </tr>

                        </table>

                        <input type="hidden" name="action" value="update"/>
                        <input type="hidden" name="page_options"
                               value="o_platform_url,o_email,o_password,o_software,o_executable,o_resource,o_region,o_instance_type,o_input_type"/>
                        <p class="submit">
                            <input type="submit" class="button-primary" value="<?php _e('Save Changes') ?>"/>
                        </p>
                    </form>
                </div>
                <?php
            }

    }

    private function constructSoftwareList()
    {

        $software_id = get_option('o_software');
        if ($software_id == '') {
            $is_init_setup = true;
            $_SESSION['init_input_type'] = 'input';
        } else {
            $is_init_setup = false;
        }

        $email = get_option('o_email');
        $pass = get_option('o_password');


        $url = get_option('o_platform_url')."/jobs/softwares.xml";
        $process = curl_init($url);
        $body = array();
        curl_setopt($process, CURLOPT_HEADER, 1);
        curl_setopt($process, CURLOPT_USERPWD, $email . ":" . $pass);
        curl_setopt($process, CURLOPT_TIMEOUT, 30);
       curl_setopt($process, CURLOPT_CUSTOMREQUEST, "PUT");
        curl_setopt($process, CURLOPT_RETURNTRANSFER, TRUE);
        curl_setopt($process, CURLOPT_POSTFIELDS, http_build_query($body));

        $return = curl_exec($process);
        $header_size = curl_getinfo($process, CURLINFO_HEADER_SIZE);
        $header = substr($return, 0, $header_size);
        $body = substr($return, $header_size);
        curl_close($process);

        $xml = simplexml_load_string($body);

        $html = "<tr> <th>Software</th> <td> <select class=\"id-selector\" id=\"o_software_list\" name=\"o_software_list\">";

        foreach ($xml->{'software'} as $software) {
            $name = (string)($software->name);
            $id = (string)($software->id);

            if($is_init_setup == true) {
                $_SESSION['init_soft_id'] = $id;
                $software_id = $id;
                $is_init_setup = false;
            }

            if ((string)$software->id == $software_id) {

                $html .= "<option value = \"" . $name . "\" data-id=" . $id . " selected>" . $name . "</option>";

            } else {

                $html .= "<option  value = \"" . $name . "\" data-id=" . $id . ">" . $name . "</option>";
            }
        }

        $html .= "</select> </td> </tr>";

        echo $html;
    }

    private function getAvailableExecutables($software_id) {

        $email = get_option('o_email');
        $pass = get_option('o_password');
        $response_data = Array(); //"id" -> "name"
        $url = get_option('o_platform_url')."/executables.xml";

        $process = curl_init($url);

        curl_setopt($process, CURLOPT_HEADER, 1);
        curl_setopt($process, CURLOPT_USERPWD, $email . ":" . $pass);
        
        curl_setopt($process, CURLOPT_TIMEOUT, 30);
        curl_setopt($process, CURLOPT_RETURNTRANSFER, TRUE);

        $return = curl_exec($process);
        $header_size = curl_getinfo($process, CURLINFO_HEADER_SIZE);
        $header = substr($return, 0, $header_size);
        $body = substr($return, $header_size);
        curl_close($process);

        $xml = simplexml_load_string($body);

        foreach ($xml->{'executable'} as $executable) {



            if ((string)$executable->{'software-id'} == $software_id) {

                $name = (string)($executable->name);
                $id = (string)($executable->id);
                $response_data[$id] = $name;
            }
        }

        return $response_data;

    }

    private function constructExecutableList()
    {
        $software_id = get_option('o_software');
        if ($software_id == '') {
            $software_id = $_SESSION['init_soft_id'];
            $executable_id = '';
            $is_init_setup = true;
        } else {
            $is_init_setup = false;
            $executable_id = get_option('o_executable');
        }

        $availableExec = $this->getAvailableExecutables($software_id);

        $html = "<tr> <th>Executable</th> <td> <select class=\"id-selector\" id=\"o_executable_list\" name=\"o_executable_list\">";


        foreach ($availableExec as $id => $name) {

            if($is_init_setup == true) {
                $_SESSION['init_exec_id'] = $id;
                $executable_id = $id;
                $is_init_setup = false;
            }

            if ($id == $executable_id) {

                $html .= "<option value = \"" . $name . "\" data-id=" . $id . " selected>" . $name . "</option>";

            } else {

                $html .= "<option  value = \"" . $name . "\" data-id=" . $id . ">" . $name . "</option>";
            }
        }

        $html .= "</select> </td> </tr>";
        echo $html;
    }

    private function getAvailableResources($software_id) {

        $email = get_option('o_email');
        $pass = get_option('o_password');

        $response_data = Array(); //"id" -> "name"
        $url = get_option('o_platform_url')."/jobs/resources.xml";

        $body = Array("software_id" => $software_id);

        $request = curl_init($url);

        curl_setopt($request, CURLOPT_USERPWD, $email . ":" . $pass);
        curl_setopt($request, CURLOPT_CUSTOMREQUEST, "PUT");
        curl_setopt($request, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($request, CURLOPT_POSTFIELDS, http_build_query($body));

        $return = curl_exec($request);

        $xml = simplexml_load_string($return);

        if(curl_errno($request) == 0){

            foreach ($xml->{'resource'} as $resource) {

                $response_data[(string)($resource->id)] = (string)($resource->name);
            }

        } else {

            $response_data['curlerror'] = curl_getinfo($request);
        }

        curl_close($request);

        return $response_data;
    }

    private function constructResourseList()
    {
        $software_id = get_option('o_software');
        if ($software_id == '') {
            $software_id = $_SESSION['init_soft_id'];
            $resource_id = '';
            $is_init_setup = true;
        } else {
            $software_id = get_option('o_software');
            $is_init_setup = false;
            $resource_id = get_option('o_resource');
        }

        $availableResources = $this -> getAvailableResources($software_id);

        $html = "<tr> <th>Resource</th> <td> <select class=\"id-selector\" id=\"o_resources_list\" name=\"o_resources_list\">";


        foreach ($availableResources as $id => $name) {

            if($is_init_setup == true) {
                $_SESSION['init_res_id'] = $id;
                $resource_id = $id;
                $is_init_setup = false;
            }

            if ($id == $resource_id) {

                $html .= "<option value = \"" . $name . "\" data-id=" . $id . " selected>" . $name . "</option>";

            } else {

                $html .= "<option  value = \"" . $name . "\" data-id=" . $id . ">" . $name . "</option>";
            }
        }

        $html .= "</select> </td> </tr>";


        echo $html;

    }

    private function getAvailableRegions($software_id, $resource_id) {
        $email = get_option('o_email');
        $pass = get_option('o_password');

        $response_data = Array(); //"id" -> "name"
        $url = get_option('o_platform_url')."/jobs/regions.xml";

        $body = Array("software_id" => $software_id, "resource_id" => $resource_id);

        $request = curl_init($url);

        curl_setopt($request, CURLOPT_USERPWD, $email . ":" . $pass);
        curl_setopt($request, CURLOPT_CUSTOMREQUEST, "PUT");
        curl_setopt($request, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($request, CURLOPT_POSTFIELDS, http_build_query($body));

        $return = curl_exec($request);

        $xml = simplexml_load_string($return);

        if(curl_errno($request) == 0){

            foreach ($xml->{'region'} as $region) {

                $response_data[(string)($region->id)] = (string)($region->name);
            }

        } else {

            $response_data['curlerror'] = curl_getinfo($request);
        }

        curl_close($request);

        return $response_data;

    }

    private function constructRegionList()
    {
        $region_id = '';
        $software_id = get_option('o_software');

        if ($software_id == '') {
            $software_id = $_SESSION['init_soft_id'];
            $resource_id = $_SESSION['init_res_id'];;
            $is_init_setup = true;
        } else {
            $software_id = get_option('o_software');
            $resource_id = get_option('o_resource');
            $region_id = get_option('o_region');
            $is_init_setup = false;
        }

        $availableRegions = $this -> getAvailableRegions($software_id, $resource_id);


        $html = "<tr> <th>Region</th> <td> <select class=\"id-selector\" id=\"o_regions_list\" name=\"o_regions_list\">";

        foreach ($availableRegions as $id => $name) {

            if($is_init_setup == true) {
                $_SESSION['init_reg_id'] = $id;
                $region_id = $id;
                $is_init_setup = false;
            }

            if ($id == $region_id) {

                $html .= "<option value = \"" . $name . "\" data-id=" . $id . " selected>" . $name . "</option>";

            } else {

                $html .= "<option  value = \"" . $name . "\" data-id=" . $id . ">" . $name . "</option>";
            }
        }

        $html .= "</select> </td> </tr>";


        echo $html;

    }

    private function getAvailableITypes($software_id, $region_id) {
        $email = get_option('o_email');
        $pass = get_option('o_password');

        $response_data = Array(); //"id" -> "name"
        $url = get_option('o_platform_url')."/jobs/instance_types.xml";

        $body = Array("software_id" => $software_id, "region_id" => $region_id);

        $request = curl_init($url);

        curl_setopt($request, CURLOPT_USERPWD, $email . ":" . $pass);
        curl_setopt($request, CURLOPT_CUSTOMREQUEST, "PUT");
        curl_setopt($request, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($request, CURLOPT_POSTFIELDS, http_build_query($body));

        $return = curl_exec($request);

        $xml = simplexml_load_string($return);

        if(curl_errno($request) == 0){

            foreach ($xml->{'instance-type'} as $instance_type) {

                $response_data[(string)($instance_type->id)] = (string)($instance_type->name);
            }

        } else {

            $response_data['curlerror'] = curl_getinfo($request);
        }

        curl_close($request);

        return $response_data;

    }

    private function constructInstanceTypeList()
    {
        $itype_id = '';
        $software_id = get_option('o_software');

        if ($software_id == '') {
            $software_id = $_SESSION['init_soft_id'];
            $region_id = $_SESSION['init_reg_id'];;
            $is_init_setup = true;
        } else {
            $software_id = get_option('o_software');
            $region_id = get_option('o_region');
            $itype_id = get_option('o_instance_type');
            $is_init_setup = false;
        }

        $availableITypes = $this -> getAvailableITypes($software_id, $region_id);


        $html = "<tr> <th>Instance type</th> <td> <select class=\"id-selector\" id=\"o_instance_type_list\" name=\"o_instance_type_list\">";

        foreach ($availableITypes as $id => $name) {

            if($is_init_setup == true) {
                $_SESSION['init_itype_id'] = $id;
                $itype_id = $id;
                $is_init_setup = false;
            }

            if ($id == $itype_id) {

                $html .= "<option value = \"" . $name . "\" data-id=" . $id . " selected>" . $name . "</option>";

            } else {

                $html .= "<option  value = \"" . $name . "\" data-id=" . $id . ">" . $name . "</option>";
            }
        }

        $html .= "</select> </td> </tr>";

        echo $html;

    }

    public function enqueue_styles()
    {
        wp_register_style('bs_mod', plugin_dir_url( __FILE__ ) . '../public/css/tw-bs.3.1.1.css', false, '1.0.0');
        wp_enqueue_style( 'bs_mod');

        wp_enqueue_style($this->plugin_name, plugin_dir_url(__FILE__) . 'css/cloudsme-prototype-admin.css', array(), $this->version, 'all');

    }

    public function enqueue_scripts()
    {
        //add_action( 'wp_enqueue_scripts', array($this, 'settings_selection_script' ));
        //add_action( 'wp_ajax_settings_selection', array($this, 'settings_selection_func' ));
        //wp_enqueue_script($this->plugin_name, plugin_dir_url(__FILE__) . 'js/settings_selection.js', array('jquery'), $this->version, false);
       // wp_enqueue_script($this->plugin_name, plugin_dir_url(__FILE__) . 'js/cloudsme-prototype-admin.js', array('jquery'), $this->version, false);
        wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . '../public/js/bootstrap.min.js', false, $this->version, false );

    }

    private function cleanOptions(){
        delete_option("o_software");
        delete_option("o_executable");
        delete_option("o_resource");
        delete_option("o_region");
        delete_option("o_instance_type");
        delete_option("o_input_type");

    }

    private function cleanCreds(){
        delete_option("o_platform_url");
        delete_option("o_email");
        delete_option("o_password");

    }
}

?>